import src.jadecobra.aws_lambda.deploy.lambda_function
import src.jadecobra.toolkit
import os

class TestAwsDeployLambdaFunction(src.jadecobra.tester.TestCase):

    def test_deploy_lambda_function(self):
        with self.assertRaises(TypeError):
            src.jadecobra.aws_lambda.deploy.lambda_function.LambdaFunction(
                function_name='bob_function'
            )
            os.remove('bob_function.zip')

    def test_deploy_lambda_function_attributes(self):
        self.assert_attributes_equal(
            src.jadecobra.aws_lambda.deploy.lambda_function.LambdaFunction,
            [
                '__class__',
                '__delattr__',
                '__dict__',
                '__dir__',
                '__doc__',
                '__eq__',
                '__format__',
                '__ge__',
                '__getattribute__',
                '__getstate__',
                '__gt__',
                '__hash__',
                '__init__',
                '__init_subclass__',
                '__le__',
                '__lt__',
                '__module__',
                '__ne__',
                '__new__',
                '__reduce__',
                '__reduce_ex__',
                '__repr__',
                '__setattr__',
                '__sizeof__',
                '__str__',
                '__subclasshook__',
                '__weakref__',
                'delete_directory',
                'delete_zipfile',
                'delimiter',
                'directory',
                'package_code',
                'python_filename',
                's3_key',
                'update_lambda_code',
                'upload_to_s3',
                'zip_filename'
            ]
        )


import datetime
import os


class Environment(object):

    def __init__(self, environment):
        self.environment = self.get_environment(environment)

    def environment_name(self):
        return self.environment.name()

    @classmethod
    def name_tag(cls, name):
        return name if name else cls.name

    @staticmethod
    def region():
        return 'us-west-2'

    @staticmethod
    def today():
        return f"{datetime.datetime.today().date()}"

    @staticmethod
    def aws_profile():
        return os.environ.get('AWS_PROFILE', 'DEV').lower()

    @staticmethod
    def arn(value):
        return f'arn:aws:{value}'

    def iam_role_arn(self, account=None, role_name=None):
        return self.arn(f'iam::{account}:role/{role_name}')

    def account_id(self):
        return self.environment.account_id()

    def vpc_id(self):
        return self.environment.vpc_id()

    def aws_environment(self):
        return dict(
            account=self.account_id(),
            region=self.region()
        )

    def get_environment(self, environment):
        '''Return Environment Module based on AWS_PROFILE given in Command Line'''
        return dict(
            dev=environment.dev,
            tst=environment.tst,
            prd=environment.prod,
        ).get(
            self.aws_profile(), environment.dev
        )

    @staticmethod
    def base_cidr():
        return '10.0.0.0/8'


import argparse
from . import lambda_function
from . import lambda_layer

def main():
    parser = argparse.ArgumentParser(
        description="Update Existing Lambda Functions or Publish Layers"
    )
    parser.add_argument(
        "-f",
        "--deploy_function",
        help="Package and Deploy a Lambda Function to S3. Update the Lambda Function if it exists",
    )
    parser.add_argument(
        "-l", "--publish_layer",
        nargs="+",
        help="Package and Publish Lambda Layer(s)"
    )
    parser.add_argument("-b", "--bucket_name", required=True)
    parser.add_argument("-p", "--profile_name")

    args = parser.parse_args()

    # Add calls to respective classes
    # Implement subparsers to replace if statements
    if args.deploy_function:
        print(f"\tCreating Package for {args.deploy_function}")
        lambda_function.LambdaFunction(
            args.deploy_function,
            bucket_name=args.bucket_name,
            profile_name=args.profile_name,
        )
    if args.publish_layer:
        print(
            f'\tCreating Lambda Layer for dependencies: {", ".join(args.publish_layer)}'
        )
        lambda_layer.LambdaLayer(
            args.publish_layer,
            bucket_name=args.bucket_name,
            profile_name=args.profile_name,
        )


if __name__ == '__main__':
    main()


import src.jadecobra
import src.jadecobra.tester
import src.jadecobra.toolkit
import jadecobra.versioning
import os


class TestJadeCobraToolkit(src.jadecobra.tester.TestCase):

    def test_toolkit(self):
        self.assert_attributes_equal(
            src.jadecobra.toolkit,
            [
                '__builtins__',
                '__cached__',
                '__doc__',
                '__file__',
                '__loader__',
                '__name__',
                '__package__',
                '__spec__',
                'datetime',
                'delete',
                'delimiter',
                'error',
                'file_exists',
                'get_commit_message',
                'header',
                'log_performance',
                'logger',
                'make_dir',
                'os',
                'pathlib',
                'time',
                'time_it',
                'to_camel_case',
                'write_config',
                'write_file'
            ]
        )

    def test_to_camel_case(self):
        self.assertEqual(
            src.jadecobra.toolkit.to_camel_case('abc-def-hij'),
            'AbcDefHij'
        )


import datetime
import pathlib
import time
import os

def logger(message, level="INFO"):
    print(f"[{level}] {message}")

def error(message):
    logger(message, level="ERROR")

def delete(filepath):
    try:
        os.remove(filepath)
    except FileNotFoundError:
        f"Could not find {filepath}"

def file_exists(filepath):
    return pathlib.Path(filepath).exists()

def make_dir(filepath):
    try:
        os.makedirs(pathlib.Path(filepath).parent)
    except FileExistsError:
        pass

def write_config(filepath=None, parser=None):
    "Write Config Parameters to filepath"
    make_dir(filepath)
    with open(filepath, "w") as configfile:
        parser.write(configfile)

def write_file(filepath=None, data=None):
    "Write Credentials to filepath"
    filepath = filepath.replace('\\', '/')
    make_dir(filepath)
    logger(f"Writing data to {filepath}")
    with open(filepath, "w") as writer:
        writer.write(str(data))

def delimiter():
    print("="*80)

def header(environment):
    delimiter()
    print(f"\t[WARNING] You are making changes to the {environment} Environment [WARNING]")
    delimiter()

def log_performance(message):
    performance = f"{datetime.datetime.now()}:{message}\n"
    logs_path = 'tests/logs'
    os.makedirs(logs_path, exist_ok=True)
    with open(f'{logs_path}/performance.log', 'a') as writer:
        writer.write(performance)
    print(performance)

def time_it(*args, function=None, description='run process', **kwargs):
    start_time = time.time()
    result = function(*args, **kwargs)
    log_performance(f'{description}:{time.time() - start_time:.1f}')
    return result

def to_camel_case(text):
    return ''.join(text.title().split('-'))

def get_commit_message():
    return input("Enter commit message: ")


import src.jadecobra.aws_lambda
import src.jadecobra.toolkit


class TestAwsLambda(src.jadecobra.tester.TestCase):

    def test_aws_lambda(self):
        self.assert_attributes_equal(
            src.jadecobra.aws_lambda,
            [
                '__builtins__',
                '__cached__',
                '__doc__',
                '__file__',
                '__loader__',
                '__name__',
                '__package__',
                '__path__',
                '__spec__',
                'deploy',
                'get_arn',
            ]
        )

    def test_get_lambda_function_arn(self):
        region = "region"
        account = "012345678901"
        name = "lambda_function_name"
        self.assertEqual(
            src.jadecobra.aws_lambda.get_arn(
                region=region,
                account=account,
                name=name
            ),
            f"arn:aws:lambda:{region}:{account}:function:{name}"
        )


import src.jadecobra.tester
import os

class TestZBuildDeploy(src.jadecobra.tester.TestCase):

    def test_z_build_and_publish(self):
        self.build_and_publish()

    def test_z_published_version_is_test_version(self):
        os.system('pip install jadecobra')
        import jadecobra
        self.assertEqual(
            jadecobra.__version__,
            src.jadecobra.__version__
        )


import src.jadecobra.aws_environment
import src.jadecobra.toolkit


class TestAwsEnvironment(src.jadecobra.tester.TestCase):

    def test_environment_attributes(self):
        self.assert_attributes_equal(
            src.jadecobra.aws_environment.Environment,
            [
                '__class__',
                '__delattr__',
                '__dict__',
                '__dir__',
                '__doc__',
                '__eq__',
                '__format__',
                '__ge__',
                '__getattribute__',
                '__getstate__',
                '__gt__',
                '__hash__',
                '__init__',
                '__init_subclass__',
                '__le__',
                '__lt__',
                '__module__',
                '__ne__',
                '__new__',
                '__reduce__',
                '__reduce_ex__',
                '__repr__',
                '__setattr__',
                '__sizeof__',
                '__str__',
                '__subclasshook__',
                '__weakref__',
                'account_id',
                'arn',
                'aws_environment',
                'aws_profile',
                'base_cidr',
                'environment_name',
                'get_environment',
                'iam_role_arn',
                'name_tag',
                'region',
                'today',
                'vpc_id'
            ]
        )


import shutil
import zipfile
import os

from . import deploy_lambda


class LambdaFunction(deploy_lambda.LambdaDeployer):

    def __init__(self, function_name: str = None, bucket_name=None, profile_name=None):
        super().__init__(bucket_name=bucket_name, profile_name=profile_name)
        self.function_name = function_name
        self.package_code()
        self.upload_to_s3()
        self.update_lambda_code()
        self.delimiter()
        self.delete_zipfile()

    @staticmethod
    def directory():
        return "lambda_functions"

    def zip_filename(self):
        return f"{self.function_name}.zip"

    def python_filename(self):
        return f"{self.function_name}.py"


    def package_code(self):
        # add everything in the folder except files/directories named tests
        self.delimiter()
        print(f"\tzipping up {self.function_name} ...")
        with zipfile.ZipFile(self.zip_filename(), "w", compresslevel=9) as compressor:
            for filename in os.listdir():
                if "tests" in filename or filename.endswith(".zip") or "scent.py" in filename:
                    continue
                else:
                    compressor.write(filename)

    def update_lambda_code(self):
        self.delimiter()
        message = f'\tupdating {self.function_name} in {self.environment_name}::'
        try:
            self.lambda_client.update_function_code(
                FunctionName=self.function_name,
                S3Bucket=self.s3_bucket,
                S3Key=self.s3_key(),
                Publish=True,
            )
        except self.lambda_client.exceptions.ClientError as error:
            print(f"{message}:{error}")
        else:
            print(f"{message}:success")



import os
import shutil

from . import deploy_lambda


class LambdaLayer(deploy_lambda.LambdaDeployer):

    def __init__(
        self,
        dependencies=None,
        bucket_name=None,
        profile_name="DEV",
        name=None
    ):
        super().__init__(bucket_name=bucket_name, profile_name=profile_name)
        self.dependencies = dependencies
        self.set_name(name)
        self.delete_directory()
        self.package_layer()
        # self.upload_to_s3()
        # self.delete_previous_layer_version()
        # self.publish_layer()
        # self.delimiter()
        # self.delete_directory()
        # self.delete_zipfile()

    def set_name(self, name):
        try:
            self.name = self.dependencies[0] if name is None else name
        except IndexError:
            self.name = self.directory()

    def publish_layer(self):
        self.delimiter()
        message = f"Publishing {self.name} Layer::"
        try:
            self.lambda_client.publish_layer_version(
                LayerName=self.name,
                CompatibleRuntimes=["python3.6", "python3.7", "python3.8", "python3.9"],
                Content={
                    "S3Bucket": self.s3_bucket,
                    "S3Key": self.s3_key(),
                },
            )
        except (
            self.lambda_client.exceptions.ServiceException,
            self.lambda_client.exceptions.ResourceNotFoundException,
            self.lambda_client.exceptions.TooManyRequestsException,
            self.lambda_client.exceptions.InvalidParameterValueException,
            self.lambda_client.exceptions.CodeStorageExceededException,
        ) as error:
            print(f"{message}FAILED::{error}")
        else:
            print(f"{message}Success")

    def directory(self):
        return f"lambda_layers/{self.name}"

    def runtime(self):
        return "python"

    def install_dependencies(self):
        try:
            for dependency in self.dependencies:
                os.system(
                    f"pip install --target ./{self.directory()}/{self.runtime()} {dependency} --upgrade --platform linux_x86_64 --only-binary=:all: --platform linux_aarch64 --only-binary=:all:"
                )
        except TypeError:
            print("No dependencies to install")

    def package_layer(self):
        self.delimiter()
        print(f"Creating Archive for {self.name} Layer")
        os.system("python -m pip install -U pip")
        self.install_dependencies()
        shutil.make_archive(
            self.name, "zip",
            root_dir=self.directory(),
            # base_dir=self.runtime()
        )

    def zip_filename(self):
        return f"{self.name}.zip"

    def s3_key(self):
        return f"{self.directory()}/{self.zip_filename()}"

    def get_layer_versions(self):
        try:
            return (
                layer["Version"]
                for layer in self.lambda_client.list_layer_versions(
                    LayerName=self.name
                )["LayerVersions"]
            )
        except (
            self.lambda_client.exceptions.ServiceException,
            self.lambda_client.exceptions.InvalidParameterValueException,
            self.lambda_client.exceptions.ResourceNotFoundException,
            self.lambda_client.exceptions.TooManyRequestsException,
            KeyError,
        ) as error:
            print(f"Layer Creation FAILED::{error}")

    def delete_previous_layer_version(self):
        for version in self.get_layer_versions():
            self.lambda_client.delete_layer_version(
                LayerName=self.name, VersionNumber=version
            )
            print(f"Layer Version: {version} deleted in {self.environment_name}")



import src.jadecobra
import src.jadecobra.tester


class TestJadeCobra(src.jadecobra.tester.TestCase):

    def test_jadecobra(self):
        self.assert_attributes_equal(
            src.jadecobra,
            [
                '__builtins__',
                '__cached__',
                '__doc__',
                '__file__',
                '__loader__',
                '__name__',
                '__package__',
                '__path__',
                '__spec__',
                '__version__',
                'aws_environment',
                'aws_lambda',
                'tester',
                'toolkit',
                'versioning'
            ]
        )


import boto3
import os
import shutil


class LambdaDeployer:

    def __init__(self, profile_name=None, bucket_name=None, region="us-west-2"):
        print('-'*70)
        print(f'currently in {os.getcwd()}')
        print('-'*70)
        self.environment_name = profile_name
        self.s3_bucket = bucket_name
        self.region = region
        self.session = boto3.session.Session(
            region_name=self.region,
            profile_name=profile_name,
        )
        self.s3 = self.session.client("s3")
        self.lambda_client = self.session.client("lambda")

    def delimiter(self):
        print("=" * 100)

    def s3_key(self):
        return f"{self.directory()}/{self.zip_filename()}"

    def upload_to_s3(self):
        self.delimiter()
        print(
            f"Uploading {self.zip_filename()} to S3://{self.s3_bucket}/{self.directory()}..."
        )
        with open(self.zip_filename(), "rb") as data:
            self.s3.upload_fileobj(data, self.s3_bucket, self.s3_key())

    def delete_directory(self):
        return shutil.rmtree(self.directory(), ignore_errors=True)

    def delete_zipfile(self):
        return os.remove(self.zip_filename())



import src.jadecobra
import src.jadecobra.tester


class TestJadeCobraTester(src.jadecobra.tester.TestCase):

    def test_tester_attributes(self):
        self.assert_attributes_equal(
            src.jadecobra.tester,
            [
                'TestCase',
                '__builtins__',
                '__cached__',
                '__doc__',
                '__file__',
                '__loader__',
                '__name__',
                '__package__',
                '__spec__',
                'json',
                'os',
                'shutil',
                'subprocess',
                'toolkit',
                'unittest',
                'versioning'
            ]
        )

    def test_tester_test_case_attributes(self):
        self.assert_attributes_equal(
            src.jadecobra.tester.TestCase,
            [
                '__call__',
                '__class__',
                '__delattr__',
                '__dict__',
                '__dir__',
                '__doc__',
                '__eq__',
                '__format__',
                '__ge__',
                '__getattribute__',
                '__getstate__',
                '__gt__',
                '__hash__',
                '__init__',
                '__init_subclass__',
                '__le__',
                '__lt__',
                '__module__',
                '__ne__',
                '__new__',
                '__reduce__',
                '__reduce_ex__',
                '__repr__',
                '__setattr__',
                '__sizeof__',
                '__str__',
                '__subclasshook__',
                '__weakref__',
                '_addExpectedFailure',
                '_addUnexpectedSuccess',
                '_baseAssertEqual',
                '_callCleanup',
                '_callSetUp',
                '_callTearDown',
                '_callTestMethod',
                '_classSetupFailed',
                '_class_cleanups',
                '_deprecate',
                '_diffThreshold',
                '_formatMessage',
                '_getAssertEqualityFunc',
                '_truncateMessage',
                'addClassCleanup',
                'addCleanup',
                'addTypeEqualityFunc',
                'assertAlmostEqual',
                'assertAlmostEquals',
                'assertCountEqual',
                'assertDictContainsSubset',
                'assertDictEqual',
                'assertEqual',
                'assertEquals',
                'assertFalse',
                'assertGreater',
                'assertGreaterEqual',
                'assertIn',
                'assertIs',
                'assertIsInstance',
                'assertIsNone',
                'assertIsNot',
                'assertIsNotNone',
                'assertLess',
                'assertLessEqual',
                'assertListEqual',
                'assertLogs',
                'assertMultiLineEqual',
                'assertNoLogs',
                'assertNotAlmostEqual',
                'assertNotAlmostEquals',
                'assertNotEqual',
                'assertNotEquals',
                'assertNotIn',
                'assertNotIsInstance',
                'assertNotRegex',
                'assertNotRegexpMatches',
                'assertRaises',
                'assertRaisesRegex',
                'assertRaisesRegexp',
                'assertRegex',
                'assertRegexpMatches',
                'assertSequenceEqual',
                'assertSetEqual',
                'assertTrue',
                'assertTupleEqual',
                'assertWarns',
                'assertWarnsRegex',
                'assert_',
                'assert_attributes_equal',
                'assert_cdk_templates_equal',
                'build_and_publish',
                'countTestCases',
                'create_cdk_templates',
                'debug',
                'defaultTestResult',
                'doClassCleanups',
                'doCleanups',
                'enterClassContext',
                'enterContext',
                'fail',
                'failIf',
                'failIfAlmostEqual',
                'failIfEqual',
                'failUnless',
                'failUnlessAlmostEqual',
                'failUnlessEqual',
                'failUnlessRaises',
                'failureException',
                'id',
                'longMessage',
                'maxDiff',
                'read_json',
                'remove_dist',
                'run',
                'setUp',
                'setUpClass',
                'shortDescription',
                'skipTest',
                'subTest',
                'tearDown',
                'tearDownClass'
            ]
        )



import src.jadecobra.aws_lambda.deploy.deploy_lambda
import src.jadecobra.tester

class TestAwsLambdaTools(src.jadecobra.tester.TestCase):

    def test_lambda_deployer(self):
        src.jadecobra.aws_lambda.deploy.deploy_lambda.LambdaDeployer()

    def test_lambda_deployer_attributes(self):
        self.assert_attributes_equal(
            src.jadecobra.aws_lambda.deploy.deploy_lambda.LambdaDeployer,
            [
                '__class__',
                '__delattr__',
                '__dict__',
                '__dir__',
                '__doc__',
                '__eq__',
                '__format__',
                '__ge__',
                '__getattribute__',
                '__getstate__',
                '__gt__',
                '__hash__',
                '__init__',
                '__init_subclass__',
                '__le__',
                '__lt__',
                '__module__',
                '__ne__',
                '__new__',
                '__reduce__',
                '__reduce_ex__',
                '__repr__',
                '__setattr__',
                '__sizeof__',
                '__str__',
                '__subclasshook__',
                '__weakref__',
                'delete_directory',
                'delete_zipfile',
                'delimiter',
                's3_key',
                'upload_to_s3'
            ]
        )



import json
import os
import shutil
import subprocess
import unittest

from . import versioning
from . import toolkit


class TestCase(unittest.TestCase):

    maxDiff = None

    @staticmethod
    def read_json(filepath):
        '''Return a dictionary from a json file'''
        with open(f'{filepath}.template.json') as template:
            return json.load(template)

    def create_cdk_templates(self):
        '''Create CloudFormation using CDK with presets'''
        result = toolkit.time_it(
            (
                'cdk ls '
                '--no-version-reporting '
                '--no-path-metadata '
                '--no-asset-metadata'
            ),
            function=subprocess.run,
            description=f'cdk ls',
            shell=True,
            capture_output=True,
        )
        print(result.stderr.decode())
        print(result.stdout.decode())
        self.assertEqual(result.returncode, 0)

    def assert_cdk_templates_equal(self, stack_name):
        '''Check if stack_name in cdk.out folder and tests/fixtures are the same'''
        self.assertEqual(
            self.read_json(f"cdk.out/{stack_name}"),
            self.read_json(f"tests/fixtures/{stack_name}")
        )

    def assert_attributes_equal(self, thing=None, attributes=None):
        '''Check that the given attributes match the attributes of thing'''
        self.assertEqual(
            sorted(dir(thing)), sorted(attributes)
        )

    @staticmethod
    def remove_dist():
        '''Remove Dist folder for new distribution'''
        try:
            shutil.rmtree('dist')
        except FileNotFoundError:
            'already removed'

    def build_and_publish(self):
        '''Build the python distribution and upload to pypi'''
        versioning.Version().git_push()
        self.remove_dist()
        os.system('python3 -m build')
        self.assertEqual(
            os.system('python3 -m twine upload dist/*'), 0
        )


import src.jadecobra.tester
import src.jadecobra.versioning


class TestJadeCobraVersioning(src.jadecobra.tester.TestCase):

    def test_version_attributes(self):
        self.assert_attributes_equal(
            src.jadecobra.versioning,
            [
                'Version',
                '__builtins__',
                '__cached__',
                '__doc__',
                '__file__',
                '__loader__',
                '__name__',
                '__package__',
                '__spec__',
                'os',
                're',
                'toolkit',
            ]
        )

    def test_version_class_attributes(self):
        self.assert_attributes_equal(
            src.jadecobra.versioning.Version,
            [
                '__class__',
                '__delattr__',
                '__dict__',
                '__dir__',
                '__doc__',
                '__eq__',
                '__format__',
                '__ge__',
                '__getattribute__',
                '__getstate__',
                '__gt__',
                '__hash__',
                '__init__',
                '__init_subclass__',
                '__le__',
                '__lt__',
                '__module__',
                '__ne__',
                '__new__',
                '__reduce__',
                '__reduce_ex__',
                '__repr__',
                '__setattr__',
                '__sizeof__',
                '__str__',
                '__subclasshook__',
                '__weakref__',
                'get_pyproject_version',
                'git_push',
                'pyproject',
                'read_pyproject',
                'semantic_version_pattern',
                'update_module_version',
                'update_pyproject_version'
            ]
        )

    def test_update_version(self):
        version = src.jadecobra.versioning.Version()
        self.assertEqual(
            version.get_pyproject_version(),
            ('0.3.', '1'),
        )



import re
import os

from . import toolkit


class Version(object):

    def __init__(self):
        self.text = self.read_pyproject()
        self.version, self.patch = self.get_pyproject_version()
        self.new_version = f"{self.version}{int(self.patch)+1}"

    @staticmethod
    def pyproject():
        return 'pyproject.toml'

    @staticmethod
    def semantic_version_pattern():
        return r'version\s+=\s+"(\d+[.]\d+[.])(\d+)"'

    def read_pyproject(self):
        '''Return contents of file'''
        with open(self.pyproject()) as file:
            return file.read()

    def get_pyproject_version(self):
        '''Get version from pyproject.toml'''
        return re.search(
            self.semantic_version_pattern(),
            self.text
        ).group(1, 2)

    def update_pyproject_version(self):
        '''Update version in pyproject.toml'''
        toolkit.write_file(
            filepath=self.pyproject(),
            data=re.sub(
                self.semantic_version_pattern(),
                f'version = "{self.new_version}"',
                self.text
            )
        )

    def update_module_version(self):
        '''Update Module Version'''
        toolkit.write_file(
            filepath='src/jadecobra/__init__.py',
            data=f'__version__ = "{self.new_version}"',
        )

    def git_push(self):
        '''Update version for project and push upstream'''
        try:
            self.update_pyproject_version()
            self.update_module_version()
        except FileNotFoundError:
            pass
        finally:
            os.system(f'git commit -am "{toolkit.get_commit_message()}"')
            os.system('git push')
